#!/usr/bin/perl
### mai_prep.pl


$] >= 5.004 or die "Perl version must be >= 5.004 (Currently $]).\n";

use Env qw(INT_SCR INT_BIN MAI_SCR);
use lib "$INT_SCR";  #### Location of Generic.pm
use Generic;

###Usage info/check

sub Usage{

`$MAI_SCR/pod2man_mai.pl  $MAI_SCR/mai_prep.pl`;
exit 1;
}
@ARGV == 1 or Usage();
@args = @ARGV;

$prefix       = shift;


$infile     = "$prefix.slc";
$roiin      = "$prefix"."_roi.in";
$roimaiin   = "$prefix"."_roi_mai.in";
$roimaiout  = "$prefix"."_roi_mai.out";
$outfilef   = "$prefix"."_f.slc";
$outfileb   = "$prefix"."_b.slc";


##########################
Message "Checking I/O";
##########################
@Infiles    = ("$infile.rsc");
@Outfiles   = ($roimaiin, "$outfilef.rsc", "$outfileb.rsc");
IOcheck(\@Infiles, \@Outfiles);
Log("mai_prep.pl", @args);


##########################################
Message "Reading roi input file: $roiin";
##########################################
$rawinfile 	= `$MAI_SCR/ReadRoiIn.pl $roiin "Master input file"`;
$debug 		= `$MAI_SCR/ReadRoiIn.pl $roiin "Debug flag"`;
$width 		= `$MAI_SCR/ReadRoiIn.pl $roiin "Bytes per input line"`;
$xmax  		= `$MAI_SCR/ReadRoiIn.pl $roiin "Good bytes per input line"`;
$proc_start 	= `$MAI_SCR/ReadRoiIn.pl $roiin "First line to read"`;
$number_of_patches = `$MAI_SCR/ReadRoiIn.pl $roiin "Number of patches"`;
$xmin       	= `$MAI_SCR/ReadRoiIn.pl $roiin "First sample pair to use"`;
$patch_size     = `$MAI_SCR/ReadRoiIn.pl $roiin "Azimuth Patch Size"`;
$valid_samples  = `$MAI_SCR/ReadRoiIn.pl $roiin "Number of valid pulses"`;
$deskew         = `$MAI_SCR/ReadRoiIn.pl $roiin "Deskew?"`;
$Caltone        = `$MAI_SCR/ReadRoiIn.pl $roiin "Caltone location"`;

$aa  		= `$MAI_SCR/ReadRoiIn.pl $roiin "Start range bin, number to process" 2`;
($first_range_bin, $out_pixel) = split /\s+/, $aa;

$aa 		= `$MAI_SCR/ReadRoiIn.pl $roiin "Master file Doppler centroid coefs" 4`;
($dop0, $dop1, $dop2, $dop3)     = split /\s+/, $aa;

$iusedop       	= `$MAI_SCR/ReadRoiIn.pl $roiin "Doppler average method"`;
$earth_radius   = `$MAI_SCR/ReadRoiIn.pl $roiin "Earth radius"`;
$velocity       = `$MAI_SCR/ReadRoiIn.pl $roiin "Body fixed S/C velocities"`;
$height         = `$MAI_SCR/ReadRoiIn.pl $roiin "Spacecraft height"`;
$PLANET_GM      = `$MAI_SCR/ReadRoiIn.pl $roiin "Planet GM"`;
$AntennaSide    = `$MAI_SCR/ReadRoiIn.pl $roiin "Left, Right or Unknown Pointing"`;

$aa		= `$MAI_SCR/ReadRoiIn.pl $roiin "SCH Velocity Vector 1" 3`;
($VelocitySCH[0], $VelocitySCH[1], $VelocitySCH[2]) = split /\s+/, $aa;

$aa 		= `$MAI_SCR/ReadRoiIn.pl $roiin "SCH Acceleration Vector 1" 3`;
($AccelerationSCH[0], $AccelerationSCH[1], $AccelerationSCH[2]) = split /\s+/, $aa;

$starting_rng   = `$MAI_SCR/ReadRoiIn.pl $roiin "Range of first sample in data file"`;
$prf      	= `$MAI_SCR/ReadRoiIn.pl $roiin "PRF"`;

$aa 		= `$MAI_SCR/ReadRoiIn.pl $roiin "i/q means" 2`;
($i_bias, $q_bias) =  split /\s+/, $aa;

$flipiq       	= `$MAI_SCR/ReadRoiIn.pl $roiin "Flip i/q?"`;
$sl_azim_res    = `$MAI_SCR/ReadRoiIn.pl $roiin "Azimuth resolution"`;
$nazlooks      	= `$MAI_SCR/ReadRoiIn.pl $roiin "Number of azimuth looks"`;
$sampling_freq  = `$MAI_SCR/ReadRoiIn.pl $roiin "Range sampling rate"`;
$chirp_slope    = `$MAI_SCR/ReadRoiIn.pl $roiin "Range chirp slope"`;
$pulse_length   = `$MAI_SCR/ReadRoiIn.pl $roiin "Range pulse duration"`;
$near_rng_ext   = `$MAI_SCR/ReadRoiIn.pl $roiin "Range chirp extension points"`;
$sec_rng_mig    = `$MAI_SCR/ReadRoiIn.pl $roiin "Secondary range migration correction?"`;
$wavelength     = `$MAI_SCR/ReadRoiIn.pl $roiin "Radar wavelength"`;
$rng_spec_weights = `$MAI_SCR/ReadRoiIn.pl $roiin "Range spectral weighting"`;
$spec_shift_frac  = `$MAI_SCR/ReadRoiIn.pl $roiin "Spectral shift fraction"`;

$aa 		= `$MAI_SCR/ReadRoiIn.pl $roiin "Linear resampling coefs" 4`;
($sloper, $intr, $slopea, $inta) 	  = split /\s+/, $aa;

$aa		= `$MAI_SCR/ReadRoiIn.pl $roiin "Linear resampling deltas" 4`;
($dsloper, $dintr, $dslopea, $dinta) = split /\s+/, $aa;

$infileagcf    = `$MAI_SCR/ReadRoiIn.pl $roiin "AGC file"`;
$infiledwpf    = `$MAI_SCR/ReadRoiIn.pl $roiin "DWP file"`;

##########################################
Message "Computing MAI parameters";
##########################################

$DOPAV = 0;
for ($i=0;$i<$out_pixel;$i++){
	$DOPAV = $DOPAV+$dop0+$dop1*$i+$dop2*$i*$i+$dop3 *$i*$i*$i;
}
$DOPAV = $DOPAV/$out_pixel;

$frac   = 0.5;
$veleff = $velocity*sqrt($earth_radius/($earth_radius+$height));
#$AntennaLen = $sl_azim_res*2.;
#$n = abs($DOPAV*$AntennaLen*$prf/$veleff/2.0);
$azres  = $sl_azim_res/(1.0-$frac); #-$n);  
$AntennaLen = $sl_azim_res*2.6;
$dop_f  = $DOPAV+$frac*$veleff/$AntennaLen/$prf;
$dop_b  = $DOPAV-$frac*$veleff/$AntennaLen/$prf;


$c 		= 299792456.;
$pi 		= 4.*atan2(1.,1.);
$delr 		= $c/2./$sampling_freq;
$mid_rng	= $starting_rng+($out_pixel-1)*$delr/2.;
$sin_theta 	= sqrt( 1 - ($height / ($mid_rng)**2 ));  #use mid range or starting range? and does it matter? (uses DOPs anyway later on...)

$sin_squint 	= $dop_f*$prf / ( 2 * $veleff * $sin_theta ) * $wavelength; # neglect vertical velocity
$squint_rad 	= atan2( $sin_squint, sqrt( 1 - $sin_squint ** 2 ) );
$squint_deg_f 	= 180 * $squint_rad / $pi;

$sin_squint 	= $dop_b*$prf / ( 2 * $veleff * $sin_theta ) * $wavelength; # neglect vertical velocity
$squint_rad 	= atan2( $sin_squint, sqrt( 1 - $sin_squint ** 2 ) );
$squint_deg_b 	= 180 * $squint_rad / $pi;


$r01 = $starting_rng+($first_range_bin-$near_rng_ext-1)*$delr;
$velfile = "get_time.in";

open (TEMP,">$velfile") or die "Can't write to $velfile\n";
print TEMP <<END;
$height
$starting_rng
$VelocitySCH[0] $VelocitySCH[1] $VelocitySCH[2]
$AccelerationSCH[0] $AccelerationSCH[1] $AccelerationSCH[2] 
$first_range_bin
$near_rng_ext
$AntennaSide
$earth_radius
$sampling_freq	
$wavelength
$dop_f $dop_b
$prf
$PLANET_GM
END
close(TEMP);

$vels = "velocities.rsc";
`rm $vels`;
`$MAI_SCR/get_time < $velfile > $vels`;

$veleff_real = Use_rsc "$vels read VELEFF_F";

$delta_time_f = -$wavelength*$r01*$prf*$dop_f/2./$veleff_real/$veleff_real;

$first_line_utc		= Use_rsc "$infile read 		FIRST_LINE_UTC";
$center_line_utc	= Use_rsc "$infile read 		CENTER_LINE_UTC";
$last_line_utc		= Use_rsc "$infile read 		LAST_LINE_UTC";

$first_line_utcf	= $first_line_utc  + $delta_time_f;
$center_line_utcf	= $center_line_utc + $delta_time_f;
$last_line_utcf		= $last_line_utc   + $delta_time_f;

$delta_time_b = -$wavelength*$r01*$prf*$dop_b/2./$veleff_real/$veleff_real;

$first_line_utcb	= $first_line_utc  + $delta_time_b;
$center_line_utcb	= $center_line_utc + $delta_time_b;
$last_line_utcb		= $last_line_utc   + $delta_time_b;



##########################################
Message "Writing resource files: $outfilef.rsc, $outfileb.rsc";
##########################################
chomp($dir1=`pwd`);
`cp $dir1/$infile.rsc $dir1/$outfilef.rsc`;
$outfile = $outfilef;

Use_rsc "$outfile write DOPPLER_RANGE0           $dop_f";  
Use_rsc "$outfile write DOPPLER_RANGE1           0";    
Use_rsc "$outfile write DOPPLER_RANGE2           0";   
Use_rsc "$outfile write DOPPLER_RANGE3           0";   
Use_rsc "$outfile write SQUINT		         $squint_deg_f";   
Use_rsc "$outfile write FIRST_LINE_UTC	         $first_line_utcf";   
Use_rsc "$outfile write CENTER_LINE_UTC	         $center_line_utcf";   
Use_rsc "$outfile write LAST_LINE_UTC	         $last_line_utcf";   
Use_rsc "$outfile write DELTA_TIME	         $delta_time_f";   



Use_rsc "$outfile write AZIMUTH_RESOLUTION       $azres";  #NOTE azimuth resolution not equal azimuth pixel size  
`cp $dir1/$outfilef.rsc $dir1/$outfileb.rsc`;

$outfile = $outfileb;
Use_rsc "$outfile write DOPPLER_RANGE0           $dop_b";  
Use_rsc "$outfile write DOPPLER_RANGE1           0";    
Use_rsc "$outfile write DOPPLER_RANGE2           0";   
Use_rsc "$outfile write DOPPLER_RANGE3           0";   
Use_rsc "$outfile write SQUINT		         $squint_deg_b";   
Use_rsc "$outfile write FIRST_LINE_UTC	         $first_line_utcb";   
Use_rsc "$outfile write CENTER_LINE_UTC	         $center_line_utcb";   
Use_rsc "$outfile write LAST_LINE_UTC	         $last_line_utcb";   
Use_rsc "$outfile write DELTA_TIME	         $delta_time_b";   


#########################################
Message "Writing roi_mai input file: ${prefix}_roi_mai.in";
#########################################

open (ROI,">$roimaiin") or die "Can't write to $roimaiin\n";

print ROI <<END;

RDF File for ROI           
Master input file                                (-) = $rawinfile                         ! Raw data for channel 1
Second input file                                (-) = /dev/null  		       ! Raw data for channel 2
Master SLC                                       (-) = $outfilef                        ! SLC channel 1
Second SLC                                       (-) = $outfileb                       ! SLC channel 2
Eight look image file                            (-) = 8lk                             ! Eight look image file
Debug flag                                       (-) = 0                               ! 
Bytes per input line                             (-) = $width    $width                ! files 1 and 2
Good bytes per input line                        (-) = $xmax $xmax                     ! including header, files 1 and 2
First line to read                               (-) = $proc_start                           ! 
Number of patches                                (-) = $number_of_patches              ! org 40, 1 for testing
First sample pair to use                         (-) = $xmin                           ! 
Number of valid pulses                           (-) = $valid_samples                  ! 
Azimuth Patch Size                               (-) = $patch_size                     !
Deskew?                                          (-) = $deskew                               ! 
Caltone location                                 (-) = 0 0                             !  % of sample rate, files 1 and 2
Start range bin, number to process               (-) = $first_range_bin $out_pixel     ! org 6626, test with 1600
Delta azimuth, range pixels for second file      (-) = 0 0                    ! 
Master file Doppler centroid coefs               (-) = $dop_f 0. 0. 0.         ! (Hz/prf) quadratic polynomial
Second file Doppler centroid coefs               (-) = $dop_b 0. 0. 0.         ! (Hz/prf) quadratic polynomial
Original Doppler centroid coefs                  (-) = $dop0 $dop1 $dop2 $dop3         ! (Hz/prf) quadratic polynomial
Doppler average method                           (-) = 0                               ! 1: use file 1 dopp, 2: file 2, 3: avg
Earth radius                                     (m) = $earth_radius                   ! 
Body fixed S/C velocities                      (m/s) = $velocity $velocity             ! files 1, 2
Spacecraft height                                (m) = $height $height                 ! files 1, 2
Planet GM					 (-) = $PLANET_GM
Left, Right or Unknown Pointing 		 (-) = $AntennaSide
SCH Velocity Vector 1			       (m/s,m/s,m/s) = $VelocitySCH[0] $VelocitySCH[1] $VelocitySCH[2]
SCH Velocity Vector 2			       (m/s,m/s,m/s) = $VelocitySCH[0] $VelocitySCH[1] $VelocitySCH[2]
SCH Acceleration Vector 1		       (-,-,-) = $AccelerationSCH[0] $AccelerationSCH[1] $AccelerationSCH[2] 
SCH Acceleration Vector 2		       (-,-,-) = $AccelerationSCH[0] $AccelerationSCH[1] $AccelerationSCH[2]



Range of first sample in data file               (m) = $starting_rng $starting_rng     ! files 1, 2
PRF                                              (-) = $prf $prf                       ! pps, files 1, 2
i/q means                                        (-) = $i_bias $q_bias $i_bias $q_bias ! file 1 (i,q), file 2 (i,q)
Flip i/q?                                        (-) = n                               ! 
Azimuth resolution                               (m) = $azres                    ! 
Number of azimuth looks                          (-) = 4                               ! 
Range sampling rate                             (Hz) = $sampling_freq                  ! 
Range chirp slope                                (-) = $chirp_slope                    ! Hz/s
Range pulse duration                             (s) = $pulse_length                   ! 
Range chirp extension points                     (-) = $near_rng_ext                   ! 
Secondary range migration correction?            (-) = n                               ! 
Radar wavelength                                 (m) = $wavelength                     ! 
Range spectral weighting                         (-) = 1.                              ! 
Spectral shift fraction                          (-) = 0. 0.                           ! % of range/azimuth bandwidth to remove
Linear resampling coefs                          (-) = 0 0 0 0                         ! sloper,  intr,  slopea,  inta 
Linear resampling deltas                         (-) = 0 0 0 0                         ! dsloper, dintr, dslopea, dinta
AGC file                                         (-) = $infileagcf                    ! AGC file name
DWP file                                         (-) = $infiledwpf                    ! DWP file name
END
close(ROI);
###############################################################################
###############################################################################

exit 0;

=pod

=head1 USAGE

B<mai_prep.pl> I<date> 
  
=head1 FUNCTION

prepares input files for roi_mai 

=head1 HISTORY

Perl  Script : Noa BD Bechor, 01/02/2008, modified from roi_prep.pl
Modification: Noa BD Bechor, 25/Sep/2010

=head1 LAST UPDATE

Noa BD Bechor 2010/09/25

=cut

