#!/usr/bin/perl
### process_mai.pl

use Env qw(INT_SCR INT_BIN MAI_SCR);
use lib "$INT_SCR";  #### Location of Generic.pm
use Generic;

###Use as: $MAI_SCR/process_mai.pl date1-date2.proc 
sub Usage{

`$MAI_SCR/pod2man_mai.pl  $MAI_SCR/process_mai.pl`;
exit 1;
}
@ARGV == 1 or Usage();
$processfile = shift;
##########################################################################
Message "Read the inputfile";
##########################################################################
open IN, "$processfile" or die "Can't read $processfile\n";
while (chomp($line = <IN>)){
  $line =~ /=/ or next;
  $line =~ s/\s//g; ###Remove all whitespace
  ($keyname, $value) = split /=/, $line;

  if ($value =~ /,/){ ### more than one value, i.e. models
    @values = split /,/, $value;
    $i=0;
    foreach $val (@values){
      $$keyname[$i] = $val;
      $i++;
    }
  }
  else{
    $$keyname = $value;
  }
}

##########################################################################
Message "Define Variables";
##########################################################################
chomp($dir0=`pwd`);
#### Variables that can be set in int_date1_date2.proc
$Rlooks_sim         or $Rlooks_sim         = 4;
$Rlooks_int         or $Rlooks_int         = $Rlooks_sim;
$Rlooks_unw         or $Rlooks_unw         = $Rlooks_sim;
$Rlooks_sml         or $Rlooks_sml         = 16;
$pixel_ratio        or $pixel_ratio        = 5;
$Alooks_sml         or $Alooks_sml         = $Rlooks_sml*$pixel_ratio;
$MPI_PARA           or $MPI_PARA           = "no";
$NUM_PROC           or $NUM_PROC           = 1;
$ROMIO              or $ROMIO              = "no";
$OrbitType          or $OrbitType          = "ODR";
$GeoDir             or $GeoDir             = "GEO";
$Clean		    or $Clean		   = "N";
$IonoCorr	    or $IonoCorr	   = "N";
$Deskew		    or $Deskew		   = "Y";


$im1 or $im1 = $SarDir1;
$im2 or $im2 = $SarDir2;

$im1 =~ s!.*\/!!g;
$im2 =~ s!.*\/!!g;

$im12 or $im12 = "$im1-$im2";

$slc1 = "$im1.slc";
$slc2 = "$im2.slc";

$Sim = $SimDir;
$Sim =~ s!.*\/!!g;

$MaiDir = "$IntDir"."-"."MAI";


chdir $dir0;
chdir $SarDir1;
chomp($SarDir1=`pwd`);
chdir $dir0; 
chdir $SarDir2;
chomp($SarDir2=`pwd`);
chdir $dir0;
unless(-d $IntDir) {`mkdir $IntDir`;}
chdir $IntDir;
chomp($IntDir=`pwd`);
chdir $dir0;
unless(-d $MaiDir) {`mkdir $MaiDir`;}
chdir $MaiDir;
chomp($MaiDir=`pwd`);
chdir $dir0;

if (($Clean eq "Y") || ($Clean eq "y")){
	chdir $SarDir1;
	`$MAI_SCR/clean_files_imdir.pl $im1`;
	chdir $SarDir2;
	`$MAI_SCR/clean_files_imdir.pl $im2`;
	chdir $MaiDir;
	`$MAI_SCR/clean_files_intdir.pl $im1 $im2`;
}

##############################################
Message "Processing forweard and backward slcs for date $im1";
##############################################

chdir $SarDir1;
`$MAI_SCR/mai_prep.pl $im1`;

print STDERR "Starting ROI 1 forward \n";

$roiin   = "$im1"."_roi_mai.in";
$roiout  = "$im1"."_roi_mai.out";
$outfilef = "$im1"."_f.slc";
$outfileb = "$im1"."_b.slc";

`$MAI_SCR/mai.pl      $im1    \\
                      $roiin  \\
                      $roiout \\
                      $outfilef \\
                      $outfileb`;
Status "mai.pl";


chdir $dir0;
   
##############################################
Message "Processing forweard and backward slcs for date $im2";
##############################################

chdir $SarDir2;
`$MAI_SCR/mai_prep.pl $im2`;

print STDERR "Starting ROI 2 forward \n";

$roiin   = "$im2"."_roi_mai.in";
$roiout  = "$im2"."_roi_mai.out";
$outfilef = "$im2"."_f.slc";
$outfileb = "$im2"."_b.slc";

`$MAI_SCR/mai.pl      $im2    \\
                      $roiin  \\
                      $roiout \\
                      $outfilef \\
                      $outfileb`;
Status "mai.pl";

chdir $dir0;

##############################################
Message "Linking files";
##############################################

$slc1f = "$im1"."_f.slc";
$slc1b = "$im1"."_b.slc";
$raw1  = "$im1".".raw";

$slc2f = "$im2"."_f.slc";
$slc2b = "$im2"."_b.slc";
$raw2  = "$im2".".raw";

$slcrsc1f = "$im1"."_f.slc.rsc";
$slcrsc1b = "$im1"."_b.slc.rsc";

$slcrsc2f = "$im2"."_f.slc.rsc";
$slcrsc2b = "$im2"."_b.slc.rsc";

$dopfile  = "$im1.slc.rsc";

$cullfile = "$im1"."-"."$im2"."_cull.off";
$cullrscfile = "$im1"."-"."$im2"."_cull.off.rsc";

$resampin  = "$im1"."-"."$im2"."_resamp.in";
$intrscorg ="$im1"."-"."$im2.int.rsc";

chdir $MaiDir;
Link_here "$SarDir1/$slc1f"; 
Link_here "$SarDir1/$slc1b";
Link_here "$SarDir1/$raw1";

Link_here "$SarDir2/$slc2f"; 
Link_here "$SarDir2/$slc2b";
Link_here "$SarDir2/$raw2";

Link_here "$SarDir1/$slcrsc1f";
Link_here "$SarDir1/$slcrsc1b";

Link_here "$SarDir2/$slcrsc2f";
Link_here "$SarDir2/$slcrsc2b";

Link_here "$SarDir1/$dopfile";

Link_here "$IntDir/$cullfile";
Link_here "$IntDir/$cullrscfile";
Link_here "$IntDir/$resampin";
Link_here "$IntDir/$intrscorg";

##############################################
Message "Resampling to make forward/backward interferograms";
##############################################

chdir $dir0;
chdir $MaiDir;

$sqf = "f";
$sqb = "b";


Message "$MAI_SCR/resamp_mai.pl $im1 $im2 $sqf 1 $pixel_ratio $MPI_PARA $NUM_PROC $ROMIO";
`$MAI_SCR/resamp_mai.pl $im1 $im2 $sqf 1 $pixel_ratio $MPI_PARA $NUM_PROC $ROMIO`;

Message "$MAI_SCR/resamp_mai.pl $im1 $im2 $sqb 1 $pixel_ratio $MPI_PARA $NUM_PROC $ROMIO";
`$MAI_SCR/resamp_mai.pl $im1 $im2 $sqb 1 $pixel_ratio $MPI_PARA $NUM_PROC $ROMIO`;

Status "resamp_mai.pl";


##############################################
Message "Writing $im12."."mai.rsc";
##############################################
$int_az = "$im12".".mai";
$outfilef = "$im12"."_f.int";
$outfileb = "$im12"."_b.int";

`cp $outfilef.rsc $int_az.rsc`;

Use_rsc "$int_az write MAI_FRAC         0.5";


##############################################
Message "Subtracting forward and backward interferograms";
##############################################
$width  = Use_rsc "$int_az read WIDTH";

Message "$MAI_SCR/diffigram $outfilef $outfileb $int_az $width";
`$MAI_SCR/diffigram $outfilef $outfileb $int_az $width`;


$MaiTranslate = "$im12".".mai";

##############################################
Message "Translating";
##############################################
chdir $MaiDir;
$roiin1 = "$im1"."_roi_mai.in";
$roiin2 = "$im2"."_roi_mai.in";

Link_here "$SarDir1/$roiin1";
Link_here "$SarDir2/$roiin2";

`rm ${im12}_mai_translate.in`;
Message "$MAI_SCR/translate_prep.pl $im1 $im2 $im12 $MaiTranslate";
`$MAI_SCR/translate_prep.pl $im1 $im2 $im12 $MaiTranslate`;


Message "$MAI_SCR/translate < ${im12}_mai_translate.in";
`$MAI_SCR/translate < ${im12}_mai_translate.in`;


##############################################
Message "Geocoding";
##############################################

$int_az_rmg = "$im12".".atd";
$length     = Use_rsc "$int_az read FILE_LENGTH";


#`$INT_BIN/cpx2rmg $int_az $int_az_rmg $width $length`;

chdir $dir0;
chdir $GeoDir;
chomp($GeoDir=`pwd`);

$rect = "rect_lookup.in";

chdir $dir0;
chdir $MaiDir;
Link_here "$IntDir/$rect";

$outgeocoded = "geo"."_"."$im12"."_"."MAI.hgt";
$orgrsc      = "geo"."_"."$im12".".unw.rsc";

`cp $IntDir/$orgrsc $outgeocoded.rsc`;
Use_rsc "$outgeocoded write MAI_FRAC         0.5";


$Lookup = `$MAI_SCR/ReadRoiIn.pl $rect "Longitude-Latitude Lookup File Name"`;
Link_here "$GeoDir/$Lookup";

$aa  	= `$MAI_SCR/ReadRoiIn.pl $rect "Lookup File Dimensions" 2`;
($Lwidth, $Llength) = split /\s+/, $aa;

$Lxmin = `$MAI_SCR/ReadRoiIn.pl $rect "Lookup File Start Sample for Output"`;
$Gwidth     = `$MAI_SCR/ReadRoiIn.pl $rect "Number of Samples for Output"`;
$Lymin   = `$MAI_SCR/ReadRoiIn.pl $rect "Lookup File Start Line for Output"`;
$Glength       = `$MAI_SCR/ReadRoiIn.pl $rect "Number of Lines for Output"`;
$aa = `$MAI_SCR/ReadRoiIn.pl $rect "Skip Every N Lookup points Across-Down" 2`;
($skip1, $skip2) = split /\s+/, $aa;
$Interp       = `$MAI_SCR/ReadRoiIn.pl $rect "Interpolation Method"`;

`rm ./$rect`;

open RECTIN, ">rect_lookup.in" or die "Can't write to rect_lookup.in\n";
print RECTIN <<END;
Longitude-Latitude Lookup File Name    (-)   = $Lookup   ! Lookup file
Lookup File Dimensions                 (-)   = $Lwidth $Llength     ! across ,  down
Input Image File Name                  (-)   = $int_az_rmg     ! file to be resampled
Input File Dimensions                  (-)   = $width $length       ! across , down
Output Image File Name                 (-)   = $outgeocoded     ! output filename
Lookup File Start Sample for Output    (-)   = $Lxmin           ! pixel across
Number of Samples for Output           (-)   = $Gwidth         ! number of pixels
Lookup File Start Line for Output      (-)   = $Lymin           ! start line 
Number of Lines for Output             (-)   = $Glength         ! number of lines
Skip Every N Lookup points Across-Down (-)   = $skip1 $skip2        ! across , down
File Type                              (-)   = RMG         ! [RMG,CPX]
Interpolation Method                   (-)   = $Interp   ! [Bilinear,Sinc,NN]
END
close(RECTIN);

Message "$INT_BIN/rect_lookup rect_lookup.in";
`$INT_BIN/rect_lookup rect_lookup.in`;
Status "rect_lookup";
`$INT_SCR/length.pl $outgeocoded`;
Status "length.pl";


##############################################
Message "Done MAI processing";
##############################################
exit 0;

=pod

=head1 USAGE

B<process_mai.pl> I<process_infile> 

process_infile: the same file as for the InSAR ROI_PAC processor (eg int.proc)

 Potential Additions:
   Deskew=Y   	for fewer phase gradients, reduced phase noise  
   		Use this option with the ROI_PAC deskew patch 
		This option needs to be consistent with the InSAR processing
		(MAI image will be produced whether you use deskew or not)
		
   Clean=Y    	for running the MAI processing from scratch each time.  
  
=head1 FUNCTION

Uses the ROI_PAC InSAR analysis parameters and raw data to generate a corresponding geocoded MAI interferogram. 

=head1 HISTORY

Perl  Script : Noa BD Bechor, 01/02/2008, modified from process_2pass.pl
Modification: Noa BD Bechor, 25/Sep/2010

=head1 LAST UPDATE

Noa BD Bechor 2010/09/25

=cut

























