#!/usr/bin/perl
###resamp_mai.pl 

$] >= 5.004 or die "Perl version must be >= 5.004 (Currently $]).\n";

use Env qw(INT_SCR INT_BIN MAI_SCR);
use lib "$INT_SCR";  #### Location of Generic.pm
use Generic;

###Usage info/check
sub Usage{

`$MAI_SCR/pod2man_mai.pl  $MAI_SCR/resamp_mai.pl`;
exit 1;
}
@ARGV >= 3 or Usage();
@args = @ARGV;
 
$date1    = shift;
$date2    = shift;
$sq       = shift;
$look_r   = shift or $look_r = 1;
$look_z   = shift or $look_z = $look_r*5;  # return to *4 after run
$MPI_PARA   = shift;
$NUM_PROC   = shift;
$ROMIO      = shift;
$file1    = "$date1"."_"."$sq".".slc";
$file2    = "$date2"."_"."$sq".".slc";
$cullfile = "$date1-${date2}_cull.off";
$intfile  = "$date1-$date2"."_"."$sq".".int";
$ampfile  = "$date1-$date2"."_"."$sq".".amp";
$resampin  = "$date1-${date2}_resamp.in";

#################
Message "Checking I/O";
#################
@Infiles  = ($file1, "$file1.rsc", 
             $file2, "$file2.rsc",
	     "$cullfile.rsc");
@Outfiles = ($intfile, "$intfile.rsc", 
             $ampfile, "$ampfile.rsc");
&IOcheck(\@Infiles, \@Outfiles);
Log("resamp.pl", @args);


##########################################
Message "Reading input file $resampin";
##########################################
$cullfile 	= `$MAI_SCR/ReadRoiIn.pl $resampin  "Image Offset File Name"`; 
$numcoef 	= `$MAI_SCR/ReadRoiIn.pl $resampin  "Number of Fit Coefficients"`;
$width1 	= `$MAI_SCR/ReadRoiIn.pl $resampin  "Number of Range Samples Image 1"`;
$width2		= `$MAI_SCR/ReadRoiIn.pl $resampin  "Number of Range Samples Image 2"`;
$aa 		= `$MAI_SCR/ReadRoiIn.pl $resampin  "Starting Line, Number of Lines, and First Line Offset" 3`;
($new_ymin,    $new_length, $first_line_offset) = split /\s+/, $aa;

$aa 		= `$MAI_SCR/ReadRoiIn.pl $resampin  "Doppler Cubic Fit Coefficients - PRF Units" 4`;
($dop0, $dop1, $dop2, $dop3)  =  split /\s+/, $aa;

$wavelength 	= `$MAI_SCR/ReadRoiIn.pl $resampin  "Radar Wavelength"`;
$rng_pixel_size	= `$MAI_SCR/ReadRoiIn.pl $resampin  "Slant Range Pixel Spacing"`;
$aa		= `$MAI_SCR/ReadRoiIn.pl $resampin  "Number of Range and Azimuth Looks" 2`;
($look_r, $look_z) =  split /\s+/, $aa;


#############################################
Message "Writing resource files: $intfile.rsc, $ampfile.rsc";
#############################################
`cp "$date1-$date2.int.rsc" $intfile.rsc`;
$intfileorg = "$date1-$date2.int";

$DOPP0    	= Use_rsc "$file1 read DOPPLER_RANGE0";
$DOPP1    	= Use_rsc "$file1 read DOPPLER_RANGE1";
$DOPP2    	= Use_rsc "$file1 read DOPPLER_RANGE2";
$DOPP3    	= Use_rsc "$file1 read DOPPLER_RANGE3";
$SQUINT   	= Use_rsc "$file1 read SQUINT";
$AZRES		= Use_rsc "$file1 read AZIMUTH_RESOLUTION";

Use_rsc  "$intfile write DOPPLER_RANGE0 	$DOPP0";
Use_rsc  "$intfile write DOPPLER_RANGE1 	$DOPP1";
Use_rsc  "$intfile write DOPPLER_RANGE2 	$DOPP2";
Use_rsc  "$intfile write DOPPLER_RANGE3 	$DOPP3";
Use_rsc  "$intfile write SQUINT		 	$SQUINT";
Use_rsc  "$intfile write AZIMUTH_RESOLUTION     $AZRES";

`cp $intfile.rsc $ampfile.rsc`;

##################################################
Message "Writing resamp input file: $date1-${date2}_resamp_mai.in";
##################################################

open RESAMP, ">$date1-${date2}_resamp_mai.in" or die "Can't write to $date1-${date2}_resamp.in\n";
print RESAMP <<END;

Image Offset File Name                                  (-)     = $cullfile
Display Fit Statistics to Screen                        (-)     = No Fit Stats    
Number of Fit Coefficients                              (-)     = $numcoef
SLC Image File 1                                        (-)     = $file1
Number of Range Samples Image 1                         (-)     = $width1
SLC Image File 2                                        (-)     = $file2
Number of Range Samples Image 2                         (-)     = $width2
Output Interferogram File                               (-)     = $intfile
Multi-look Amplitude File                               (-)     = $ampfile
Starting Line, Number of Lines, and First Line Offset   (-)     = $new_ymin    $new_length $first_line_offset
Doppler Cubic Fit Coefficients - PRF Units              (-)     = $dop0 $dop1 $dop2 $dop3 
Radar Wavelength                                        (m)     = $wavelength
Slant Range Pixel Spacing                               (m)     = $rng_pixel_size
Number of Range and Azimuth Looks                       (-)     = $look_r $look_z
Flatten with offset fit?                                (-)     = No 

END
    close(RESAMP);
###################################################
    
if($MPI_PARA eq "yes" and $NUM_PROC > 1) {
  ##### use these two lines if you do not want or cannot choose processors, or
  Message  "mpirun -np $NUM_PROC $INT_BIN/mpi_resamp_roi  $date1-${date2}_resamp.in $ROMIO > $date1-${date2}_resamp.out";
  `mpirun -np $NUM_PROC $INT_BIN/mpi_resamp_roi  $date1-${date2}_resamp.in $ROMIO > $date1-${date2}_resamp.out`;
  ##### use these two lines if you do want to choose processors
  ##### you will need to have a file named machines.LINUX under your home directo
ry
  ##### that lists all the IP's of the machines you want to choose
  ##Message "mpirun -nolocal -machinefile ~/machines.LINUX -np $NUM_PROC $INT_BIN/mpi_resamp_roi $date1-${date2}_resamp.in $ROMIO > $date1-${date2}_resamp.out";
  ##`mpirun -nolocal -machinefile ~/machines.LINUX -np $NUM_PROC $INT_BIN/mpi_resamp_roi $date1-${date2}_resamp.in $ROMIO > $date1-${date2}_resamp.out`;
}
else {
  Message  "$INT_BIN/resamp_roi  $date1-${date2}_resamp_mai.in > $date1-${date2}_resamp_mai.out";
  `$INT_BIN/resamp_roi $date1-${date2}_resamp_mai.in > $date1-${date2}_resamp_mai.out`;
}
Status "resamp_roi";

exit 0;

=pod

=head1 USAGE

B<resamp_mai.pl> I<date1, date2, f/b, mpi optional inputs> 
  
=head1 FUNCTION

prepares inputs and runs resamp_roi

=head1 HISTORY

Perl  Script : Noa BD Bechor, 01/02/2008, modified from resamp.pl
Modification: Noa BD Bechor, 25/Sep/2010

=head1 LAST UPDATE

Noa BD Bechor 2010/09/25

=cut

