#!/usr/bin/perl
### translate_prep.pl


$] >= 5.004 or die "Perl version must be >= 5.004 (Currently $]).\n";

use Env qw(INT_SCR INT_BIN MAI_SCR);
use lib "$INT_SCR";  #### Location of Generic.pm
use Generic;

###Usage info/check

sub Usage{

`$MAI_SCR/pod2man_mai.pl  $MAI_SCR/translate_prep.pl`;
exit 1;
}
@ARGV == 4 or Usage();
@args = @ARGV;

$prefix1       = shift;
$prefix2       = shift;
$im12 	       = shift;
$infile        = shift;

$roimaiin1  = "$prefix1"."_roi_mai.in";
$roimaiin2  = "$prefix2"."_roi_mai.in";

$outfile    = "$im12".".atd";


##########################
Message "Checking I/O";
##########################
@Infiles    = ($roimaiin1,$roimaiin2,"$infile.rsc");
@Outfiles   = ("$outfile.rsc");
IOcheck(\@Infiles, \@Outfiles);
Log("translate_prep.pl", @args);


##########################################
Message "Reading roi input file: $roimaiin1";
##########################################
$patch_size     = `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "Number of valid pulses"`;
$number_of_patches = `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "Number of patches"`;

$aa  		= `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "Start range bin, number to process" 2`;
($first_range_bin1, $out_pixel1) = split /\s+/, $aa;
$aa  		= `$MAI_SCR/ReadRoiIn.pl $roimaiin2 "Start range bin, number to process" 2`;
($first_range_bin2, $out_pixel2) = split /\s+/, $aa;

$starting_rng1   = `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "Range of first sample in data file"`;
$starting_rng2   = `$MAI_SCR/ReadRoiIn.pl $roimaiin2 "Range of first sample in data file"`;

$near_rng_ext   = `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "Range chirp extension points"`;
$sampling_freq  = `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "Range sampling rate"`;
$wavelength     = `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "Radar wavelength"`;

$velocity1       = `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "Body fixed S/C velocities"`;
$velocity2       = `$MAI_SCR/ReadRoiIn.pl $roimaiin2 "Body fixed S/C velocities"`;

$aa 		= `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "Master file Doppler centroid coefs" 4`;
($dopf0, $dopf1, $dopf2, $dopf3)     = split /\s+/, $aa;
$aa 		= `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "Second file Doppler centroid coefs" 4`;
($dopb0, $dopb1, $dopb2, $dopb3)     = split /\s+/, $aa;

$prf1      	= `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "PRF"`;
$prf2      	= `$MAI_SCR/ReadRoiIn.pl $roimaiin2 "PRF"`;

$height1         = `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "Spacecraft height"`;
$height2         = `$MAI_SCR/ReadRoiIn.pl $roimaiin2 "Spacecraft height"`;

$earth_radius1   = `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "Earth radius"`;
$earth_radius2   = `$MAI_SCR/ReadRoiIn.pl $roimaiin2 "Earth radius"`;

$aa		= `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "SCH Velocity Vector 1" 3`;
($VelocitySCH1[0], $VelocitySCH1[1], $VelocitySCH1[2]) = split /\s+/, $aa;
$aa		= `$MAI_SCR/ReadRoiIn.pl $roimaiin2 "SCH Velocity Vector 1" 3`;
($VelocitySCH2[0], $VelocitySCH2[1], $VelocitySCH2[2]) = split /\s+/, $aa;

$aa 		= `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "SCH Acceleration Vector 1" 3`;
($AccelerationSCH1[0], $AccelerationSCH1[1], $AccelerationSCH1[2]) = split /\s+/, $aa;
$aa 		= `$MAI_SCR/ReadRoiIn.pl $roimaiin2 "SCH Acceleration Vector 1" 3`;
($AccelerationSCH2[0], $AccelerationSCH2[1], $AccelerationSCH2[2]) = split /\s+/, $aa;

$AntennaSide    = `$MAI_SCR/ReadRoiIn.pl $roimaiin1 "Left, Right or Unknown Pointing"`;

Message "Antenna Side = $AntennaSide";

if ($AntennaSide eq "Right ") {
 	$i_lookdir = 1;
}
else{
 	$i_lookdir = 0;
}

$AzimuthLooks = Use_rsc "$infile read ALOOKS";

	
##########################################
Message "Writing resource file: $outfile.rsc";
##########################################
chomp($dir1=`pwd`);
`cp $dir1/$infile.rsc $dir1/$outfile.rsc`;

#########################################
Message "Writing translate input file: ${im12}_mai_translate.in";
#########################################
open (ROI,">${im12}_mai_translate.in") or die "Can't write to ${im12}_mai_translate.in\n";
print ROI <<END;
$infile
$outfile
$patch_size
$number_of_patches
$AzimuthLooks
$out_pixel1
$starting_rng1
$starting_rng2
$first_range_bin1
$first_range_bin2
$near_rng_ext
0
$sampling_freq
$wavelength
$velocity1
$velocity2
$dopf0 $dopf1 $dopf2 $dopf3
$dopb0 $dopb1 $dopb2 $dopb3
$prf1
$prf2
$height1
$height2
$earth_radius1
$earth_radius2
$VelocitySCH1[0] $VelocitySCH1[1] $VelocitySCH1[2]
$VelocitySCH2[0] $VelocitySCH2[1] $VelocitySCH2[2]
$AccelerationSCH1[0] $AccelerationSCH1[1] $AccelerationSCH1[2]
$AccelerationSCH1[0] $AccelerationSCH1[1] $AccelerationSCH1[2]
$i_lookdir
END
close(ROI);
###############################################################################
###############################################################################

exit 0;

=pod

=head1 USAGE

B<translate_prep.pl> I<date1, date2, im12, infile> 
  
=head1 FUNCTION

prepares inputs for mai phase to along track displacement translation

=head1 HISTORY

Perl  Script : Noa BD Bechor, 01/02/2008
Modification: Noa BD Bechor, 25/Sep/2010

=head1 LAST UPDATE

Noa BD Bechor 2010/09/25

=cut

