c     roi -- patch mode interferometric processor with prati filter 
c            wide swath version
c 
c   UPDATE LOG:
c       This version uses MPI (Message Passing Interface)
c       to parallelize the roi sequential computations.
c       Compiler flag -DMPI_PARA is needed
c       in order to have the MPI code.        03/01/04
CPOD      
CPOD=pod
CPOD
CPOD=head1 USAGE
CPOD
CPOD roi input_file 
CPOD
CPODwhere input_file is the name of the RDF formatted input file
CPOD
CPOD or 
CPOD
CPOD roi < input_file
CPOD
CPODwhere input_file is old style list of input parameters
CPOD
CPOD=head1 FUNCTION
CPOD
CPODThis program will process two raw synthetic aperture radar data sets
CPODinto images and perform a resampling operation on one of them based on input
CPODpolynomials that describe the spatially variable offset between the 
CPODtwo images. The outputs are then coaligned single-look complex (SLC) images
CPODin floating point that can be interfered to make an interferogram.
CPOD
CPODThe program forms images by an overlap save method.  The input is divided
CPODinto overlapping patches. Each patch goes through a series of filtering
CPODoperations: range compression of each line, followed by azimuth FFT, range
CPODmigration correction, azimuth compression. Resampling of the second image to 
CPODalign with the first is done during range migration correction in range, and 
CPOD with linear phase ramps in the frequency domain in azimuth.
CPOD
CPOD=head1 ROUTINES CALLED
CPOD
CPOD cfft1d_jpl, rc_patch, ac_patch, rm_patch, cffts,
CPOD radopp, moddop, outdop, nextpower,
CPOD mp_super, p_setup, mp_numthreads,
CPOD rdflen, rdfval, 
CPOD rdf_clear, rdf_init, rdf_read
CPOD
CPOD=head1 CALLED BY
CPOD
CPOD=head1 FILES USED
CPOD
CPODProgram reads in an RDF command file.
CPOD
CPODProgram reads in two flat, binary, raw data files.
CPODEach pixel in the raw data files are expected be
CPODbyte oriented data, either real (offset-video) samples,
CPODone byte each, or complex (I/Q) samples, one byte for 
CPODthe real part and one byte for the imaginary part.
CPODThe files contain the images to be interfered with each other
CPODafter processing. 
CPOD
CPODProgram reads an ASCII data window position (DWP) file and automatic 
CPODgain control (AGC) file each with format (line#,value). Zero length 
CPODis OK for these files. 
CPOD
CPOD=head1 FILES CREATED
CPOD
CPODProgram creates two SLC images as output (or one if /dev/null is specified
CPODas an output file).  It also generates a multi-look power image for diagnostic 
CPODpurposes.
CPOD
CPODThe most common usage of roi presently is to produce a single image rather than
CPODattempt to align two images with the offset fields.  This is because resamp_roi
CPODdoes a better job of handling large offset variability.
CPOD
CPOD=head1 DIAGNOSTIC FILES
CPOD
CPODCode has option to dump binary complex images after range compression,
CPODazimuth FFT, and range migration correction for channel 1
CPOD
CPOD=head1 HISTORY
CPOD
CPODOriginal Routines: Howard Zebker and Paul Rosen
CPODEdited over the years repeatedly by Paul Rosen and Scott Hensley
CPOD
CPOD=head1 LAST UPDATE
CPOD
CPODPaul Rosen, 25 Nov. 2003
CPOD
CPOD=cut

      program roi

      implicit none
#ifdef MPI_PARA
      include 'mpif.h'
#endif

      complex*8, DIMENSION(:,:), ALLOCATABLE :: trans, transb 
      complex*8, DIMENSION(:), ALLOCATABLE :: data, ref, ref1, ref2, refn2
      real*4, DIMENSION(:,:), ALLOCATABLE :: spec
      real*4       unpacki1(0:255),unpackq1(0:255)
      real*4       unpacki2(0:255),unpackq2(0:255)
      real*4	   azres, power
      real*4       kx01, kx02
      real*8       fd1, fdd1, fddd1, fdddd1, fd2, fdd2, fddd2, fdddd2
      real*8       fd3, fdd3, fddd3, fdddd3
      real*8       fdorg, fddorg, fdddorg, fddddorg
      real*8       r001, r002, r01, r02, r1, r2, r, delr
      real*8       ht1, ht2, wavl, re
      real*8       slpr, intr, slpa, inta, coefa(2)
      real*8       sloper, interr, slopea, intera
      real*8       dsloper, dinterr, dslopea, dintera
      real*8       dp1, dp2, fr1, fr2, slope1, slope2
      real*8       r_platvel1(3), r_platacc1(3), gm, earthgm
      real*8       r_platvel2(3), r_platacc2(3)
      character*64 name_in_1,name_in_2,name_out,name_outp, name_8lk 
      character*64 agcfile, dwpfile,leftright
      character*1  deskew, iqflip, srm
      integer*4    ideskew, iflip, iq, nr_fftf, nr_fftr, npow, lastnb
      integer*4    i, j, k, l, linend, line
      integer*4    nbytes1,nbytes2, ngood1, ngood2, ifirstline
      integer*4    iflag, isave, ifirst, na_valid, npatches
      integer*4    nl, nls, ifrst, irec, npts
      integer*4    ier, ipatch, lun, ifs, nlooks, nextend
      integer*4    icaltone1, icaltone2, npfin1, npfin2
      integer*4    k1start, k1end, k2start, k2end
      integer*4    iazdelta, iradelta, nlinesaz, iusedopp, nnn
      real*4       caltone1, caltone2, win1, phase, t, t0, ts, fs
      real*4       wgt, delw, rc1, rc2, a21, a22, a41, a42
      real*4       omega, omega0, omega1, phase1, phase2
      real*4       fd, t1, sum1, sum2, sum3, gcal, rhww, rhww1
      real*4       fwidth, pi, pi2
      real*4       pctbw, pctbwaz, dxsamp1, dxsamp2
      real*4       pulsedur, slope, xmi1, xmq1, xmi2, xmq2
      real*4       prf1, prf2, vel1, vel2, veleff
      integer*4    FDSC1, FDSC2, FDSC3, FDSC4, FDSC5
      integer*4    FDSC6, FDSC7, FDSC8, FDSC9, fdscrc1, fdscrc2
      integer*4    ioutrec1, ioutrec2,icnt
      integer*4    ioutrc1, ioutrc2
      integer*4    nint, NextPower
      real*4       agc(2,200000),dwp(2,1000)
      integer*4    nagc, ndwp, i_lrl
      real*4 t2, t3, t4
      character*4  ROMIO

#ifdef MPI_PARA
cc--   variables for MPI parallel code
      integer MAX_PROC
      parameter(MAX_PROC=2048)
      integer ierror
      integer*4 mpiMyNode, mpiNumNodes, num_nodes_used
      integer*4 fdsc
      integer(KIND=MPI_OFFSET_KIND) view_disp
      complex*8, DIMENSION(:), ALLOCATABLE :: tmp_buf 
      character*64 tmp_file, tmp_str
      real*4 t103, t104
      integer complex_size
      integer p_patches(MAX_PROC,2)   ! patches processed by each node
      integer i_x, i_y, residual, n_patches_p 
#endif

#ifndef RDFFAILS
      character*255 a_rdtmp
      character*255 rdfname
      character*255 rdfval
      integer*4 rdflen
#endif
      integer*4 irdf, iargc

#ifdef PARALLELIZED_ROI
      integer*4 nt, mp_numthreads, p_setup

      external mp_numthreads, p_setup
#endif

      real*4 seconds
      external seconds

      data         earthgm/0.398600485042958863d15/

cc--   MPI setup
#ifdef MPI_PARA
      call MPI_INIT(ierror)
      if (ierror .ne. 0) then
         print *,'Error - Cannot initialize MPI!'
         stop
      end if

      call MPI_COMM_RANK(MPI_COMM_WORLD, mpiMyNode, ierror)
      call MPI_COMM_SIZE(MPI_COMM_WORLD, mpiNumNodes, ierror)

cc      write(6,*) 'XXX mpiMyNode, mpiNumNodes: ', mpiMyNode, mpiNumNodes
#endif

c     **********  input parameters  **********
c     
c     enter input parameters
c
      write(6,*) 'ROI - Repeat Orbit Interferometric processor'
      write(6,*) ''
c     
      irdf = 0
#ifndef RDFFAILS
#ifdef MPI_PARA
cc      write(6,*) '***** mpiMyNode: ',mpiMyNode, ' iargc() ', iargc()
      irdf = 1
      if(mpiMyNode .eq. 0) then
         call getarg(1,rdfname)
cc         write(6,*) '***** iargc() ', iargc()
         ROMIO = "no"
         call getarg(2,ROMIO)
cc         write(6,*) '***** rdfname ', rdfname, mpiMyNode
cc         write(6,*) '***** ROMIO ', ROMIO, mpiMyNode
      end if
      call MPI_BCAST(rdfname,255,MPI_CHARACTER,0,MPI_COMM_WORLD,ierror)
      call MPI_BCAST(ROMIO,4,MPI_CHARACTER,0,MPI_COMM_WORLD,ierror)
cc      write(6,*) '***** ', mpiMyNode, ', rdfname ', rdfname
cc      write(6,*) '***** ', mpiMyNode, ', ROMIO ', ROMIO

      call rdf_init('ERRFILE=SCREEN')
      call rdf_init('ERROR_SCREEN=ON')
      call rdf_init('ERROR_OUTPUT=rdf_errors.log')
      call rdf_init('COMMENT= ! ! !')
      call rdf_read(rdfname)
#else

      if(iargc() .eq. 1) then
         irdf = 1
         call getarg(1,rdfname)
         call rdf_init('ERRFILE=SCREEN')
         call rdf_init('ERROR_SCREEN=ON')
         call rdf_init('ERROR_OUTPUT=rdf_errors.log')
         call rdf_init('COMMENT= ! ! !')
         call rdf_read(rdfname)
      end if
#endif
#endif

      if(irdf .eq. 0) then

         include 'roi.input.f'
         
#ifndef RDFFAILS
      else
         
         include 'roi.rdfinput.f'
#endif
      end if

c read in agc file and dwp file if they exist

      nagc = 0
      if (agcfile(1:9).ne.'/dev/null') then
         call rdf_getlun(lun)
         lun=30
         open(lun,file=agcfile)
         do i = 1, 200000
            read(lun,*,end=778) agc(1,i), agc(2,i)
            nagc = nagc + 1
         enddo
      endif
 778  close(lun)
! bjs - above looks like bug
! bjs - unitialized lun is CLOSEd if agcfile(1:9).eq.'/dev/null'

      ndwp = 0
      if (dwpfile(1:9).ne.'/dev/null') then
         call rdf_getlun(lun)
         lun=32
         open(lun,file=dwpfile)
         do i = 1, 20000
            read(lun,*,end=779) dwp(1,i), dwp(2,i)
            ndwp = ndwp + 1
         enddo
      endif
 779  close(lun)
! bjs - same bug here, see above      

      write(6,*) 'nagc, ndwp ', nagc, ndwp

c     compute the azimuth parameters

      dxsamp1 = vel1/prf1
      dxsamp2 = vel2/prf2
      write(6,*) 'actual orbital pulse spacing ', dxsamp1, dxsamp2

      veleff = vel1 * sqrt(re/(re+ht1))

      delr=299792456./fs/2.

      write(6,*) ''
      write(6,*) 'First  input data file ',name_in_1
      write(6,*) 'Second input data file ',name_in_2
      write(6,*) 'Output interferogram   ',name_out
      write(6,*) 'Output amplitudes      ',name_outp
      write(6,*) 'First line to read in file 1 (start at 0) ', ifirstline 
      write(6,*) '# of range input patches ', npatches
      write(6,*) 'First sample pair to use ',ifirst
      write(6,*) 'Azimuth patch size ',nnn
      write(6,*) 'Number of valid points in azimuth ',na_valid
      write(6,*) 'Deskew the image? ',deskew
      write(6,*) 'First range bin to save in file 1 ', isave
      write(6,*) 'Number of range bins to process ', nlinesaz
      write(6,*) 'Delta azimuth for second file ',iazdelta
      write(6,*) 'Delta range pixels for second file: ',iradelta
      write(6,*) 'Caltone location 1,2 ',caltone1, caltone2
      write(6,*) 'Doppler centroid quad coef 1 (Hz/prf) ', fd1, fdd1,
     $     fddd1, fdddd1
      write(6,*) 'Doppler centroid quad coef 2 (Hz/prf) ', fd2, fdd2,
     $     fddd2, fdddd2
      write(6,*) 'Original Doppler centroid quad coef (Hz/prf) ', fdorg, fddorg,
     $     fdddorg, fddddorg
      write(6,*) 'Using Doppler flag ', iusedopp
      write(6,*) 'Effective S/C Body fixed velocity 1 (m/s) ', vel1
      write(6,*) 'Effective S/C Body fixed velocity 2 (m/s) ', vel2
      write(6,*) 'Effective Azimuth sample spacing 1 (m) ', dxsamp1
      write(6,*) 'Effective Azimuth sample spacing 2 (m) ', dxsamp2
      write(6,*) 'Earth Radius (m) ', re
      write(6,*) 'Spacecraft height 1 (m) ', ht1
      write(6,*) 'Spacecraft height 2 (m) ', ht2
      write(6,*) 'Range of first pixel in range compressed file 1 (m) ',
     $     r001 
      write(6,*) 'Range of first pixel in range compressed file 2 (m) ',
     $     r002 
      write(6,*) 'PRF 1 (pps) ', prf1
      write(6,*) 'PRF 2 (pps) ', prf2
      write(6,*) 'i/q means, i1,q1, i2,q2 ', xmi1, xmq1, xmi2, xmq2
      write(6,*) 'Flip i/q ', iqflip
      write(6,*) 'Desired azimuth resolution (m) ', azres
      write(6,*) 'Number azimuth looks (m) ', nlooks
      write(6,*) 'Range sampling rate (Hz) ', fs
      write(6,*) 'Chirp Slope (Hz/s) ', slope
      write(6,*) 'Pulse Duration (s) ', pulsedur
      write(6,*) 'Chirp extension ', nextend
      write(6,*) 'Secondary Range Correction ', srm
      write(6,*) 'Radar Wavelength (m) ', wavl
      write(6,*) 'Range Spectral Weighting ', rhww
      write(6,*) 'Fractional bandwidth to remove ', pctbw, pctbwaz
      write(6,*) 'linear resampling coefs:  sloper, intr, slopea, inta',
     .     sloper,interr,slopea, intera
      write(6,*) 'linear resampling deltas: dsloper, dintr, dslopea, dinta'
     $     ,dsloper,dinterr,dslopea, dintera

c write out agc file and dwp file if they exist
      if (agcfile(1:9).ne.'/dev/null') write(6,*) 'agc file ', agcfile     
      if (dwpfile(1:9).ne.'/dev/null') write(6,*) 'dwp file ', dwpfile     

      write(6,*) ''

      t0 = seconds(0.0)           ! start timer

      pi = 4.*atan(1.)
      pi2 = 2. * pi

      iflip = 0
      iq = 1
      if(iqflip .eq. 'Y' .or. iqflip .eq. 'y') iflip = 1
      if(iqflip .eq. 'O' .or. iqflip .eq. 'o') iq = 0

      fwidth=slope*pulsedur
      if(iq .eq. 0) then 
         npts=2.*fs*pulsedur
         ts=1./(2.*fs)
      else
         npts=fs*pulsedur
         ts=1./(fs)
      end if
      if(mod(npts,2) .eq. 0) npts=npts+1
      write(6,*) 'Pulse length in points: ',npts,ts

      if(iq .eq. 1) then
         npow = NextPower(max(ngood1+npts,ngood2+npts)/2)
         nr_fftf = 2 ** npow
         nr_fftr = nr_fftf
      else
         npow = NextPower(max(ngood1+npts,ngood2+npts))
         nr_fftf = 2 ** npow
         nr_fftr = nr_fftf/2
      end if

      if (nint(2**(log(float(nnn))/log(2.))) .ne. nnn) then
         write(*,*) 'log ',log(float(nnn))/log(2.)
         stop 'Azimuth patch size not a power of 2'
      end if

      ALLOCATE( trans(nnn,nlinesaz) )
      ALLOCATE( transb(nnn,nlinesaz) )
      ALLOCATE( data(nlinesaz) )
      ALLOCATE( ref(max(nr_fftf+1,nnn)) )
      ALLOCATE( ref1(max(nr_fftf+1,nnn)) )
      ALLOCATE( ref2(max(nr_fftf+1,nnn)) )
      ALLOCATE( refn2(max(nr_fftf+1,nnn)) )
      ALLOCATE( spec(max(nr_fftf+1,nnn)/32,3) )

c      initialize transforms

      call cfft1d_jpl(nnn,ref,0)
      call cfft1d_jpl(nr_fftf,ref,0)
      call cfft1d_jpl(nr_fftr,ref,0)

      write(6,*) 'fft lengths ', nnn, nr_fftf, nr_fftr

c     compute range reference function

      fd1 = fd1 * prf1
      fdd1 = fdd1 * prf1
      fddd1 = fddd1 * prf1
      fdddd1 = fdddd1 * prf1
      fd2 = fd2 * prf2
      fdd2 = fdd2 * prf2
      fddd2 = fddd2 * prf2
      fdddd2 = fdddd2 * prf2
      fdorg = fdorg * prf2
      fddorg = fddorg * prf2
      fdddorg = fdddorg * prf2
      fddddorg = fddddorg * prf2

c     save to later compute coefficients referenced to updated starting range

      fd3    = fd1
      fdd3   = fdd1
      fddd3  = fddd1
      fdddd3 = fdddd1

      write(6,*) 'Doppler coefficients referenced to range bin index: '
      write(6,*) 'fd (Hz)                ', fd1, fd2, fdorg
      write(6,*) 'd fd/dr (Hz/s)         ', fdd1, fdd2, fddorg
      write(6,*) 'd^2 fd/dr^2 (Hz/s/s)   ', fddd1,fddd2, fdddorg
      write(6,*) 'd^3 fd/dr^3 (Hz/pix^3) ', fdddd1,fdddd2, fddddorg
      
c modify dopplers as desired in iusedopp

c      call moddopp(fd1,fdd1,fddd1,fdddd1,fd2,fdd2,fddd2,fdddd2,iusedopp,iradelta)
c      call moddopp(fdorg,fddorg,fdddorg,fddddorg,fdorg,fddorg,fdddorg,fddddorg,1,iradelta)

c reference doppler coefficients to range

      call radopp(fd1,fdd1,fddd1,fdddd1,r001,delr)
      call radopp(fd2,fdd2,fddd2,fdddd2,r002,delr)
      call radopp(fdorg,fddorg,fdddorg,fddddorg,r001,delr)

      write(6,*) 'Doppler coefficients referenced to absolute range: '
      write(6,*) 'fd (Hz)                ', fd1,    fd2,    fdorg
      write(6,*) 'd fd/dr (Hz/m)         ', fdd1,   fdd2,   fddorg
      write(6,*) 'd^2 fd/dr^2 (Hz/m^2)   ', fddd1,  fddd2,  fdddorg
      write(6,*) 'd^3 fd/dr^3 (Hz/m^3)   ', fdddd1, fdddd2, fddddorg
      
      if(iusedopp .eq. 0) then
         write(6,*) 'Doppler check: ',fd1+fdd1*r001+fddd1*r001*r001+fdddd1*r001*r001*r001, 
     .     fd2+fdd2*r002+fddd2*r002*r002+fdddd2*r002*r002*r002, 
     .     fdorg+fddorg*r01+fdddorg*r001*r001+fddddorg*r001*r001*r001
      else
         r = r001 + float(iradelta)*delr
         write(6,*) 'Doppler check: ',fd1+fdd1*r001+fddd1*r001*r001+fdddd1*r001*r001*r001, 
     .     fd2+fdd2*r+fddd2*r*r+fdddd2*r*r*r 
      endif

c     update r001 and r002 to reflect the offsets given in isave,
c     iradelta, and nextend
      
      r01 = r001 + (isave-nextend-1) * delr
      r02 = r002 + (isave-nextend-1+iradelta) * delr

      write(6,*) 'updated start ranges (m) ',r01, r02

      call outdopp(fd3,fdd3,fddd3,fdddd3,r001,r01,delr)

      write(6,*) 'Doppler coefficients referenced to output range bin index: '
      write(6,*) 'fd (Hz)              ', fd3
      write(6,*) 'd fd/dr (Hz/pix)       ', fdd3
      write(6,*) 'd^2 fd/dr^2 (Hz/pix^2) ', fddd3
      write(6,*) 'd^3 fd/dr^3 (Hz/pix^3) ', fdddd3
      write(6,*) 'OUTDOP ', fd3, fdd3, fddd3, fdddd3

      r1  = r01 + delr*(nlinesaz-1)
      r2  = r02 + delr*(nlinesaz-1)
      write(6,*) 'far range 1,2 (m) ', r1,r2
      npfin1 = r1*wavl/(2.0*azres*dxsamp1)+2
      npfin2 = r2*wavl/(2.0*azres*dxsamp2)+2
      write(6,*) 'filter points in far range ', npfin1,npfin2

      a21 = -2.0*pi/(dxsamp1*float(nnn)) ! for deskew
      a22 = -2.0*pi/(dxsamp2*float(nnn)) ! for deskew
      a41 = wavl/(2.0*azres*dxsamp1) ! for np
      a42 = wavl/(2.0*azres*dxsamp2) ! for np
      write(6,*) 'coefficient for filter points a4 ',a41, a42
      write(6,*) 'near range chirp rate ', -2. * (vel1*sqrt(re/(re+ht1)))**2
     $     /(wavl*r01), -2.* (vel2*sqrt(re/(re+ht2)))**2/(wavl*r02) 
      write(6,*) 'far  range chirp rate ', -2. * (vel1*sqrt(re/(re+ht1)))**2
     $     /(wavl*r1), -2.* (vel2*sqrt(re/(re+ht2)))**2/(wavl*r2) 
      
c secondary range migration correction to chirp rate (from CY memo)
c
      slope1 = slope
      slope2 = slope
      if((srm .eq. 'y' .or. srm .eq. 'Y')) then

         r1 = r001 + delr* nlinesaz/2.
         fr1 = -2. * vel1**2/(wavl*r1) 
         r2 = r002 + delr* nlinesaz/2.
         fr2 = -2. * vel2**2/(wavl*r2) 
         dp1 = fd1 + fdd1 * r1 + fddd1 * r1**2 + fdddd1 * r1**3
         dp2 = fd2 + fdd2 * r2 + fddd2 * r2**2 + fdddd2 * r2**3
         write(6,*) r1, r2, fr1, fr2, dp1, dp2
         slope1 = slope / (1.d0 + slope * wavl**2 * dp1**2 / fr1 /
     $        299792456.d0**2)
         slope2 = slope / (1.d0 + slope * wavl**2 * dp2**2 / fr2 /
     $        299792456.d0**2)
         
      end if
      write(6,*) 'range chirp rate and SRC corrections ', slope, slope1,
     $     slope2

      rhww1 = 1.0-rhww

      k=0
      if(pctbw.ge.0.0)then
         k1start=abs(pctbw)*npts
         k1end=npts
         k2start=0
         k2end=npts-abs(pctbw)*npts
      else
         k1start=0
         k1end=npts-abs(pctbw)*npts
         k2start=abs(pctbw)*npts
         k2end=npts
      end if
      
      write(6,*) 'npts, k1 start, end, k2 start, end'
      write(6,*) npts,k1start,k1end,k2start,k2end
      
      do i=-npts/2,npts/2
         k=k+1
         t=i*ts
         if(iq.eq.0) then
            phase = pi*slope1*t*t+pi*fs*t
            if(k.ge.k1start.and.k.le.k1end)
     +           ref1(i+npts/2+1)=cmplx(cos(phase),0.)
            phase = pi*slope2*t*t+pi*fs*t
            if(k.ge.k2start.and.k.le.k2end)
     +           ref2(i+npts/2+1)=cmplx(cos(phase),0.)
            ref(i+npts/2+1)=cmplx(cos(phase),0.)
         else
            phase = pi*slope1*t*t
            if(k.ge.k1start.and.k.le.k1end)
     +           ref1(i+npts/2+1)=cmplx(cos(phase),sin(phase))
            phase = pi*slope2*t*t
            if(k.ge.k2start.and.k.le.k2end)
     +           ref2(i+npts/2+1)=cmplx(cos(phase),sin(phase))
            ref(i+npts/2+1)=cmplx(cos(phase),sin(phase))
         end if
      end do
      
      do i=1,npts
         win1 = (rhww-rhww1*cos((2.*pi*float(i-1))/float(npts-1)))
         ref(i)=ref(i)*win1
         ref1(i)=ref1(i)*win1
         ref2(i)=ref2(i)*win1
      end do

      
      if(nextend .gt. 0) then
         do i = 1 , npts
            k = i - nextend
             if(k .le. 0) k = k + nr_fftf
            ref(k) = ref(i)
            ref1(k) = ref1(i)
            ref2(k) = ref2(i)
         end do
         do i = 1 , nextend	
       	    k = npts - nextend+i
            if(k .le. 0) k = k + nr_fftf
            ref(k) = cmplx(0.,0.)
            ref1(k) = cmplx(0.,0.)
            ref2(k) = cmplx(0.,0.)
         end do
      end if

      write(6,*) 'reference calculated in time domain.'
      
c     calculate fft of range reference function

      call cfft1d_jpl(nr_fftf,ref,-1)
      call cfft1d_jpl(nr_fftf,ref1,-1)
      call cfft1d_jpl(nr_fftf,ref2,-1)

c zero out dc and caltone location
      icaltone1=nint(caltone1*nr_fftf)
      icaltone2=nint(caltone2*nr_fftf)
      write(6,*) 'Caltone bins: ',icaltone1, icaltone2

      do i = 1, 6
         wgt = 0.5 - 0.5 * cos((i-1)/5.*pi)
         ref(i) = ref(i) * wgt
         ref1(i) = ref1(i) * wgt
         ref2(i) = ref2(i) * wgt
         ref(nr_fftf+1-i) = ref(nr_fftf+1-i) * wgt
         ref1(nr_fftf+1-i) = ref1(nr_fftf+1-i) * wgt
         ref2(nr_fftf+1-i) = ref2(nr_fftf+1-i) * wgt

         if(iq .eq. 0) then
            ref(i+nr_fftf/2) = ref(i+nr_fftf/2) * wgt
            ref1(i+nr_fftf/2) = ref1(i+nr_fftf/2) * wgt
            ref2(i+nr_fftf/2) = ref2(i+nr_fftf/2) * wgt
            ref(nr_fftf/2+1-i) = ref(nr_fftf/2+1-i) * wgt
            ref1(nr_fftf/2+1-i) = ref1(nr_fftf/2+1-i) * wgt
            ref2(nr_fftf/2+1-i) = ref2(nr_fftf/2+1-i) * wgt
            if(icaltone1 .gt. 6 .and. icaltone1 .lt. nr_fftf-6) then
               ref(i+icaltone1) = ref(i+icaltone1) * wgt
               ref1(i+icaltone1) = ref1(i+icaltone1) * wgt
               ref(icaltone1+1-i) = ref(icaltone1+1-i) * wgt
               ref1(icaltone1+1-i) = ref1(icaltone1+1-i) * wgt
               ref(i+nr_fftf-icaltone1) = ref(i+nr_fftf-icaltone1) * wgt
               ref1(i+nr_fftf-icaltone1) = ref1(i+nr_fftf-icaltone1)
     $              * wgt
               ref(nr_fftf-icaltone1+1-i) = ref(nr_fftf-icaltone1+1-i)
     $              * wgt
               ref1(nr_fftf-icaltone1+1-i) = ref1(nr_fftf-icaltone1+1-i)
     $              * wgt
            end if
            if(icaltone2 .gt. 6 .and. icaltone2 .lt. nr_fftf-6) then
               ref2(i+icaltone2) = ref2(i+icaltone2) * wgt
               ref2(icaltone2+1-i) = ref2(icaltone2+1-i) * wgt
               ref2(i+nr_fftf-icaltone2) = ref2(i+nr_fftf-icaltone2)
     $              * wgt
               ref2(nr_fftf-icaltone2+1-i) = ref2(nr_fftf-icaltone2+1-i)
     $              * wgt
            end if
         end if
      end do

c     refn2 is ref without the secondary range correction phase
c     here for debugging

      do i = 1 , nr_fftf
         refn2(i) = ref(i)
      end do

c     scale reference for channel gain, conjugate

      gcal=1./nr_fftf
      do i=1,nr_fftf
         refn2(i)=conjg(refn2(i))*gcal
         ref(i)=conjg(ref(i))*gcal
         ref1(i)=conjg(ref1(i))*gcal
         ref2(i)=conjg(ref2(i))*gcal
      end do

c     save spectra of reference functions for checking

      if(iand(iflag, 16) .eq. 16) then
         do i=1,nr_fftr/32
            sum1=0.
            sum2=0.
            sum3=0.
            do k=0,31
               sum1=sum1+cabs(ref(i*32-31+k))**2
               sum2=sum2+cabs(ref1(i*32-31+k))**2
               sum3=sum3+cabs(ref2(i*32-31+k))**2
            end do
            spec(i,1)=sum1
            spec(i,2)=sum2
            spec(i,3)=sum3
         end do
         call rdf_getlun(lun)
         lun=33
         open(lun,file='rangespecs.out')
         do i=1,nr_fftr/32
            write(lun,*) i,(spec(i,k),k=1,3)
         end do
         close(lun)
         
         call rdf_getlun(lun)
         lun=34
         open(lun,file='fullrangespecs.out')
         do i=1,nr_fftr
            write(lun,*) i,cabs(ref(i)),atan2(aimag(ref(i)),real(ref(i)))
     $           ,cabs(refn2(i)),atan2(aimag(refn2(i)),real(refn2(i))) 
         end do
         close(lun)
      end if
c     
c     open input files
c     
      call rdf_getlun(fdsc1)
      fdsc1=35
      call rdf_getlun(fdsc2)
      fdsc2=36

#ifdef MPI_PARA
cc    get num. of patches that each node would process
      do i_x = 1, MAX_PROC
        p_patches(i_x, 1) = 0
        p_patches(i_x, 2) = 0
      end do

      n_patches_p = npatches/mpiNumNodes
      residual = npatches - n_patches_p * mpiNumNodes

cc    number patches each node handles
      do i_x = 1, mpiNumNodes
        p_patches(i_x, 2) = n_patches_p
      end do

      do i_x = 1, residual
        p_patches(i_x, 2) = p_patches(i_x, 2) + 1 
      end do

cc    compute each node's starting patch
      num_nodes_used = mpiNumNodes
      do i_x = 1, mpiNumNodes
        p_patches(i_x, 1) = 0
        if (p_patches(i_x, 2) .gt. 0) then 
          if(i_x .eq. 1) then
            p_patches(i_x, 1) = 1
          else
            p_patches(i_x, 1) = p_patches(i_x-1, 1) + p_patches(i_x-1, 2)
          end if
        else
cc        count the actual num of nodes used in case there are more
cc        processors than there are patches
          num_nodes_used = num_nodes_used - 1
        end if
      end do

      if(num_nodes_used .lt. mpiNumNodes) then
        write(6,*) ' ***** Warning: You are using ', mpiNumNodes, ' processors,'
        write(6,*) '       but you only need ', num_nodes_used, ','
        write(6,*) '       because there are only this many patches. Rerun mpi_roi'
        write(6,*) '       with ', num_nodes_used, ' or less processors!'

        call MPI_FINALIZE(ierror)
        stop
      end if

cc    compute each node's ending patch
      do i_y = 1, mpiNumNodes
        p_patches(i_y, 2) = p_patches(i_y, 1) + p_patches(i_y, 2) - 1
cc        write(6,*) 'XXX node: ', i_y, 'p_start: ', p_patches(i_y, 1), 
cc     &             ', p_end: ', p_patches(i_y, 2), ' on ', mpiMyNode
      end do

      if(ROMIO .eq. "yes") then
cc--    Use ROMIO so processors can write to a file in parallel
cc      write(6,*) '***** name_out: ', name_out, mpiMyNode
      call MPI_FILE_OPEN(MPI_COMM_WORLD, name_out,
     &  MPI_MODE_WRONLY + MPI_MODE_CREATE, MPI_INFO_NULL, fdsc7,
     &  ierror)
cc      write(6,*) '***** ',mpiMyNode, ' fdsc7: ',fdsc7,' ierror ',ierror
      if(ierror .ne. 0) then
        write(6,*) '***** MPI_FILE_OPEN() failed on ', mpiMyNode,', ierror: ',ierror
        stop
      endif
      call MPI_TYPE_SIZE(MPI_COMPLEX, complex_size, ierror)
cc      write(6,*) '***** complex_size: ', complex_size, mpiMyNode
cc    the displacement of this node's starting patch from the start of the file
      view_disp = p_patches(mpiMyNode+1, 1) - 1
      view_disp = view_disp * na_valid
      view_disp = view_disp * nlinesaz
      view_disp = view_disp * complex_size
cc      write(6,*) '***** view_disp: ', view_disp, mpiMyNode

      call MPI_FILE_SET_VIEW(fdsc7,view_disp,
     &       MPI_COMPLEX,MPI_COMPLEX,'native',MPI_INFO_NULL,ierror)
      if(ierror .ne. 0) then
        write(6,*) '***** MPI_FILE_SET_VIEW() failed on ', mpiMyNode,' ierror: ',ierror
        stop
      endif

      else if (ROMIO .eq. "no") then
cc--    Use temporary files to hold partial results for each processor
      if(p_patches(mpiMyNode+1,1) .gt. 0) then
        write(tmp_str,'(i1)') mpiMyNode
        tmp_file = tmp_str(1:1) // '.slc'
        tmp_file = trim(tmp_file)
        write(6,*) '***** tmp_file : ', tmp_file
        call rdf_getlun(fdsc7)
        fdsc7=37
cc        write(6,*) '***** fdsc7 is : ', fdsc7, ' on ', mpiMyNode
      end if
      end if
#else
      call rdf_getlun(fdsc7)
      fdsc7=37
#endif

      call rdf_getlun(fdsc8)
      fdsc8=38
      call rdf_getlun(fdscrc1)
      fdscrc1=39
      call rdf_getlun(fdscrc2)
      fdscrc2=40
      write(*,*) 'units: ',fdsc1,fdsc2, fdsc7,fdsc8,fdscrc1,fdscrc2
#ifdef MPI_PARA
      if(ROMIO .eq. "yes") then
cc--    Use ROMIO so processors can read from a file in parallel
cc      write(6,*) 'XXX before MPI_FILE_OPEN, name_in_1:', name_in_1
      call MPI_FILE_OPEN(MPI_COMM_WORLD, name_in_1,
     &  MPI_MODE_RDONLY, MPI_INFO_NULL, fdsc1,
     &  ierror)
      else
cc        write(6,*) 'XXX before open, name_in_1:', name_in_1
        open(fdsc1,file=name_in_1,access='direct',recl=nbytes1,form
     $       ='unformatted')
      end if
#else
cc      write(6,*) 'XXX before open, name_in_1:', name_in_1
      open(fdsc1,file=name_in_1,access='direct',recl=nbytes1,form
     $     ='unformatted')
#endif
cbjs - quick fix for g95 and pgf95 not allowing a filename to be
cbjs -  opened more than once
      if(name_in_2(1:9).ne.'/dev/null')then
         open(fdsc2,file=name_in_2,access='direct',recl=nbytes2,form
     $       ='unformatted')
      else
         fdsc2=-2
      endif
c     
c     open output files
c     
#ifndef MPI_PARA
cc      write(6,*) '***** name_out: ', name_out
      open(fdsc7,file=name_out, recl=8*nlinesaz,access='direct',form
     $     ='unformatted')
#else
      if(ROMIO .eq. "no") then
cc        write(6,*) '***** tmp_file : ', tmp_file
        open(fdsc7,file=tmp_file, recl=8*nlinesaz,access='direct',
     $       form='unformatted')
      end if
#endif
cbjs - quick fix for g95 and pgf95 not allowing a filename to be
cbjs -  opened more than once
      if(name_outp(1:9).ne.'/dev/null')then
         open(fdsc8,file=name_outp,recl=8*nlinesaz,access='direct',form
     $        ='unformatted') 
      else
         fdsc8=-8
      endif
      if(iand(iflag,64).eq.64) then
         open(fdscrc1,file='im.rc1',recl=8*nlinesaz,access='direct',form
     $        ='unformatted')
         open(fdscrc2,file='im.rc2',recl=8*nlinesaz,access='direct',form
     $        ='unformatted') 
      end if
      write(6,*) 'Fortran Files opened.'

c offset into valid data in a patch

      ifs = (nnn-na_valid)/2

c     load the unpacking array

      if(iflip .eq. 0) then
         do i=0,255
            unpacki1(i)=float(i)-xmi1
            unpackq1(i)=float(i)-xmq1
            unpacki2(i)=float(i)-xmi2
            unpackq2(i)=float(i)-xmq2
         end do
      else
         do i=0,255
            unpacki1(i)=float(i)-xmq1
            unpackq1(i)=float(i)-xmi1
            unpacki2(i)=float(i)-xmq2
            unpackq2(i)=float(i)-xmi2
         end do
      end if
      
      ideskew = 0
      if(deskew .eq. 'y' .or. deskew .eq. 'Y') ideskew = 1

      t2=seconds(t0)
      write(6,*) 'XXX elapsed time before looping over patches', t2
c     
c     begin loop to range process data
c     
      ioutrec1 = 0
      ioutrec2 = 0
      ioutrc1 = 0
      ioutrc2 = 0

#ifdef PARALLELIZED_ROI
      nt = 1
c$    nt = mp_numthreads()
      k = p_setup(nt)
c$doacross local(k)
      do k = 0, nt-1
         call mp_super(k)
      enddo
#endif     

#ifdef DEBUG
      do ipatch = 1, 1
#else
#ifdef MPI_PARA
cc    npatches patches assigned to mpiNumNodes nodes

      do ipatch = p_patches(mpiMyNode+1,1), p_patches(mpiMyNode+1,2)
#else
      do ipatch=1,npatches
#endif
#endif
         
c     get offsets for the given patch
         
         slpr = sloper+(ipatch-1)*dsloper
         intr = interr+(ipatch-1)*dinterr
         slpa = slopea+(ipatch-1)*dslopea
         inta = intera+(ipatch-1)*dintera
         write(6,*) 'Patch ',ipatch,' resampling inputs: ',slpr,intr,slpa
     $        ,inta

         coefa(1) = slpa
         coefa(2) = inta

c convert to function of range instead of pixel

         intr = intr - slpr*r001/delr
         slpr = slpr/delr

         inta = inta - slpa*r001/delr
         slpa = slpa/delr

         write(6,*) 'Converted Resampling inputs ', slpr,intr,slpa,inta
c     
c     compress channels
         
 100     format(' processing line: ',i5,f10.3,' seconds.')

c do range compression for file 1

         if(name_out(1:9).ne.'/dev/null') then
         
         write(6,*) ''
         write(6,*) ' range compressing channel 1 .....'
         write(6,*) ''

         irec=ifirstline+(ipatch-1)*na_valid
         ifrst= ifirst+isave-1

         t3=seconds(t0)
         call rcpatch(FDSC1,nnn,nlinesaz,trans,unpacki1,unpackq1,
     $        ref1,irec,ifrst,nbytes1,ngood1,nr_fftf, nr_fftr,iq,iflip, 
     $           agc,dwp,nagc,ndwp, ROMIO) 
         t4=seconds(t0)
#ifdef MPI_PARA
         write(6,*) 'XXX rcpatch took: ',t4-t3,'s', ' on ', mpiMyNode
#else
         write(6,*) 'XXX rcpatch took: ',t4-t3,'s'
#endif

            if(iand(iflag,64) .eq. 64)then
               write(6,*) 'writing range compressed data 1 '
               do i = nnn/2-na_valid/2+1, nnn/2+na_valid/2
                  do j = 1 , nlinesaz
                     data(j) = trans(i,j)
                  end do
                  
                  ioutrc1 = ioutrc1 + 1
                  write(fdscrc1,rec=ioutrc1) (data(j),j=1,nlinesaz)
               end do
               
            end if
            
            t1=seconds(t0)
            write(6,*) 'first channel range processing elapsed time: ',t1
     $           ,'sec'

c     transform lines for file 1
         
            call cffts(trans,nnn,1,nlinesaz,nnn,1,ier)
            t1=seconds(t0)
            write(6,*) 'transformed lines ', t1, ' sec'

c     No need to repeat range compression and azimuth fft for same data
            do i = 1, nnn
               do j = 1, nlinesaz
                  transb(i,j) = trans(i,j)
               end do
            end do

            if(ipatch.eq.1 .and. (iand(iflag,8) .eq. 8))then
               write(6,*) 'writing azimuth spectrum '
               call rdf_getlun(lun)
               lun=41
               open(lun,file='im.rcas1',recl=8*nlinesaz,access='direct',form='unformatted')
               do i = 1, nnn
                  do j = 1 , nlinesaz
                     data(j) = trans(i,j)
                  end do
                  write(lun,rec=i) (data(j),j=1,nlinesaz)
               end do
               close(lun)
            end if

c     start the range migration correction for file 1
         
         write(6,*) 'start range migration correction.'
         
         nl=nlinesaz
         r=r01
            call RMpatch(trans,0.d0,0.d0,
     .           nnn,nl,nls,r,delr,wavl,vel1,ht1,re,fd1,fdd1,fddd1,fdddd1,prf1,ideskew
     $           )
            t1=seconds(t0)
            write(6,*) 'range migrated ',t1, ' sec'

            if(ipatch.eq.1 .and. (iand(iflag,4) .eq. 4))then
               write(6,*) 'writing range migrated data '
               call rdf_getlun(lun)
               lun=42
               open(lun,file='im.rcrm1',recl=8*nlinesaz,access='direct',form='unformatted')
               do i = 1, nnn
                  do j = 1 , nlinesaz
                     data(j) = trans(i,j)
                  end do
                  write(lun,rec=i) (data(j),j=1,nlinesaz)
               end do
               close(lun)
            end if

c multiply by reference and inverse transform lines for file 1
         

         nl=nlinesaz
         r = r01
         call ACpatch(trans,nnn,nl,r,delr,
     .           wavl,vel1,fd1,fdd1,fddd1,fdddd1,prf1,ht1,re,
     .           gm,r_platvel1,r_platacc1,i_lrl,
     .           npfin1,a21,a41,0.d0,0.d0,ideskew,na_valid, 
     .           fdorg, fddorg, fdddorg, fddddorg)

         t1=seconds(t0)
         write(6,*) 'Range-Doppler done',t1,' sec'

         if(ipatch.eq.1 .and. (iand(iflag,16) .eq. 16))then
               
            call rdf_getlun(lun)
            lun=43
            open(lun,file=name_8lk,access='direct',form='unformatted',recl=na_valid)

c     take looks for file 1

            icnt = 0
            do line = 1 , nlinesaz, 2
               icnt = icnt+1
               do i = 1 , na_valid/8
                  data(i) = cmplx(0.,0.)
               end do
               do j = 1 , 2
                  do i=1,na_valid/8
                     power=0.
                     do k=0,7
                        power = power+
     .                       cabs(trans(ifs+i*8-7+k,line+j-1))**2
                     end do
                     data(i)=data(i) + power
                  end do
               end do
               do i = 1 , na_valid/8
                  data(i) = cmplx(sqrt(real(data(i))),0.)
               end do
               write(lun,rec=icnt) (data(i),i=1,na_valid/8)
            end do
            close(lun)
         end if

c     if first time through, save complex images for file 1
         
         if(ipatch.eq.1 .and. (iand(iflag,1) .eq. 1))then
            write(6,*) 'writing channel 1 range doppler image'
            
            call rdf_getlun(lun)
            lun=44
            open(lun,file='trans1.dat',access='direct',form='unformatted',recl=8*nlinesaz)
            do i = 1, nnn
               do j = 1 , nlinesaz
                  data(j) = trans(i,j)
               end do
               write(lun,rec=i) (data(j),j=1,nlinesaz)
            end do
            close(lun)
         end if

c write out the slc for file 1

         t3=seconds(t0)
         do i = nnn/2-na_valid/2+1, nnn/2+na_valid/2
            do j = 1 , nlinesaz
               data(j) = trans(i,j)
cc               write(6,*) aimag(data(j)), real(data(j))
            end do
               
            ioutrec1 = ioutrec1 + 1
#ifdef MPI_PARA
          if(ROMIO .eq. "yes") then
            call MPI_FILE_WRITE(fdsc7,data,nlinesaz,MPI_COMPLEX,
     &                          MPI_STATUS_IGNORE,ierror)
cc            write(6,*) 'XXX ', mpiMyNode, ' after MPI_FILE_WRITE ',ierror
          else if(ROMIO .eq. "no") then
            write(fdsc7,rec=ioutrec1) (data(j),j=1,nlinesaz)
          end if
#else
            write(fdsc7,rec=ioutrec1) (data(j),j=1,nlinesaz)
#endif
               
         end do
         t4=seconds(t0)
#ifdef MPI_PARA
         write(6,*) 'XXX Output took : ',t4-t3,'s', ' on ', mpiMyNode
#else
         write(6,*) 'XXX Output took : ',t4-t3,'s'
#endif

         t1 = seconds(0.0)
         write(6,*) 'written up to record ',ioutrec1, ':', t1-t0, ' sec '

         end if  ! if for processing file 1

c do range migration and azimuth compression for beackward looking slc         
c     start the range migration correction 
         
         write(6,*) 'start range migration correction.'
         
         nl = nlinesaz
         r = r02
            call RMpatch(transb,
     .           slpr,intr,
     .           nnn,nl,nls,r,delr,wavl,vel2,ht2,re,fd2,fdd2,fddd2,fdddd2,prf2,ideskew
     $           )
            t1=seconds(t0)
            write(6,*) 'range migrated ',t1 ,' sec'

            if(ipatch.eq.1 .and. (iand(iflag,4) .eq. 4))then
               write(6,*) 'writing range migrated data '
               call rdf_getlun(lun)
               lun=46
               open(lun,file='im.rcrm2',access='direct',form='unformatted',recl=8*nlinesaz)
               do i = 1, nnn
                  do j = 1 , nlinesaz
                     data(j) = transb(i,j)
                  end do
                  write(lun,rec=i) (data(j),j=1,nlinesaz)
               end do
               close(lun)
            end if
 

c multiply by reference and inverse transform lines for file 2
         
c            r = r02 + (line-1)*delr
            r = r02
            call ACpatch(transb,nnn,nl,r,delr,
     .           wavl,vel2,fd2,fdd2,fddd2,fdddd2,prf2,ht2,re,
     .           gm,r_platvel2,r_platacc2,i_lrl,
     .           npfin2,a22,a42,slpa,inta,ideskew,na_valid, 
     .           fdorg, fddorg, fdddorg, fddddorg)

         t1=seconds(t0)
         write(6,*) 'Range-Doppler done',t1,' sec'
         
         if(ipatch.eq.1 .and. (iand(iflag,16) .eq. 16))then
               
            call rdf_getlun(lun)
            lun=47
            open(lun,file=name_8lk,access='direct',form='unformatted',recl=na_valid)
c     take looks for file 2

            do line = 1 , nlinesaz, 2
               icnt=icnt+1
               do i = 1 , na_valid/8
                  data(i) = cmplx(0.,0.)
               end do
               do j = 1 , 2
                  do i=1,na_valid/8
                     power=0
                     do k=0,7
                        power = power+
     .                       cabs(transb(ifs+i*8-7+k,line+j-1))**2
                     end do
                     data(i)=data(i)+power
                  end do
               end do
               do i = 1 , na_valid/8
                  data(i) = cmplx(sqrt(real(data(i))),0.)
               end do
               write(lun,rec=icnt) (data(i),i=1,na_valid/8)
            end do
         end if


c     if first time through, save complex images for file 2

         if(ipatch.eq.1 .and. (iand(iflag,2) .eq. 2))then
            write(6,*) 'writing channel 2 range doppler image'
            call rdf_getlun(lun)
            lun=48
            open(lun,file='trans2.dat',access='direct',form='unformatted',recl=8*nlinesaz)
            do i = 1, nnn
               do j = 1 , nlinesaz
                  data(j) = transb(i,j)
               end do
               write(lun,rec=i) (data(j),j=1,nlinesaz)
            end do
            close(lun)
         end if


c write out the slc for file 2

            do i = nnn/2-na_valid/2+1, nnn/2+na_valid/2
               do j = 1 , nlinesaz
                  data(j) = transb(i,j)
               end do
               ioutrec2 = ioutrec2 + 1
               write(fdsc8,rec=ioutrec2) (data(j),j=1,nlinesaz)
            end do
         t1 = seconds(0.0)
         write(6,*) 'written up to record ',ioutrec2, ':', t1-t0, ' sec '


      end do      !end patch loop
      
#ifdef MPI_PARA
      call MPI_BARRIER(MPI_COMM_WORLD, ierror)
      if(ROMIO .eq. "yes") then
        call MPI_FILE_CLOSE(fdsc7,ierror)
      else if (ROMIO .eq. "no") then
        close(fdsc7)
       if(mpiMyNode .eq. 0) then
        t103=seconds(t0)
        ALLOCATE( tmp_buf(nlinesaz) )
        call rdf_getlun(fdsc7)
        fdsc7=49
        open(fdsc7,file=name_out,recl=8*nlinesaz,access='direct',
     $       form='unformatted')

        ioutrec2 = 0
        do i = 1, mpiNumNodes
         if(p_patches(i,1) .gt. 0) then
          write(tmp_str,'(i1)') i-1
          tmp_file = tmp_str(1:1) // '.slc'
          tmp_file = trim(tmp_file)
cc          write(6,*) '***** In sequential I/O, tmp_file : ', tmp_file
          call rdf_getlun(fdsc)
          fdsc=49+i
          open(fdsc,file=tmp_file, recl=8*nlinesaz,access='direct',
     $         form='unformatted')
          ioutrec1 = 0
          do i_x = p_patches(i,1), p_patches(i,2)
          do j = nnn/2-na_valid/2+1, nnn/2+na_valid/2
            ioutrec1 = ioutrec1 + 1
            read(fdsc,rec=ioutrec1,iostat=ierror) (tmp_buf(k),k=1,nlinesaz)

            if(ierror .gt. 0) then
              write(6,*) 'XXX read error at loop i=',i
            endif

            ioutrec2 = ioutrec2 + 1
            write(fdsc7,rec=ioutrec2) (tmp_buf(k),k=1,nlinesaz)
          end do
          end do
          close(fdsc)
         end if
        end do

        close(fdsc7)
        t104=seconds(t0)
        write(6,*) '***** time for sequential read/write ', t104-t103, ' sec'
       end if
      end if
#endif

#ifdef PARALLELIZED_ROI
c$doacross local(k)
      do k = 0, nt-1
         call mp_unsuper(k)
      enddo
#endif     

      t1=seconds(t0)
      write(6,*) '***** Elapsed time =', t1, ' seconds.'

#ifdef MPI_PARA
      call MPI_BARRIER(MPI_COMM_WORLD, ierror)
      if(mpiMyNode .eq. 0) then
        t1=seconds(t0)
        write(6,*) '***** Elapsed time (parallel) =', t1, ' seconds.'
      endif

      call MPI_FINALIZE(ierror)
#endif

      end
        
      subroutine moddopp(fd1, fdd1, fddd1, fdddd1, fd2, fdd2, fddd2, fdddd2,
     .      iusedopp, iradelta)

      integer*4 iusedopp, iradelta
      real*8 fd1, fdd1, fddd1, fdddd1, fd2, fdd2, fddd2, fdddd2
      real*8 temp1, temp2, temp3, temp4

      if(iusedopp .eq. 0) then
         write(6,*) 'using doppler as is '
      elseif(iusedopp .eq. 1) then
         write(6,*) 'referencing ch 2 dopp to ch 1'
         fd2 = fd1
     .        - float(iradelta)    * fdd1 
     .        + float(iradelta)**2 * fddd1
     .        - float(iradelta)**3 * fdddd1
         fdd2 = fdd1
     .         - 2.d0 * float(iradelta)    * fddd1
     .         + 3.d0 * float(iradelta)**2 * fddd1
         fddd2 = fddd1
     .          - 3.d0 * float(iradelta)    * fdddd1
         fdddd2 = fdddd1
      elseif(iusedopp .eq. 2) then
         write(6,*) 'referencing ch 1 dopp to ch 2'
         fd1 = fd2
     .        + float(iradelta)    * fdd2 
     .        + float(iradelta)**2 * fddd2
     .        + float(iradelta)**3 * fdddd2
         fdd1 = fdd2
     .         + 2.d0 * float(iradelta)    * fddd2
     .         + 3.d0 * float(iradelta)**2 * fddd2
         fddd1 = fddd2
     .          + 3.d0 * float(iradelta)    * fdddd2
         fdddd1 = fdddd2
      elseif(iusedopp .eq. 3) then
         write(6,*) 'averaging dopplers'
         temp1 = fd2
     .        + float(iradelta)    * fdd2 
     .        + float(iradelta)**2 * fddd2
     .        + float(iradelta)**3 * fdddd2
         temp2 = fdd2
     .         + 2.d0 * float(iradelta)    * fddd2
     .         + 3.d0 * float(iradelta)**2 * fddd2
         temp3 = fddd2
     .          + 3.d0 * float(iradelta)    * fdddd2
         temp4 = fdddd2
         fd1    = (fd1    + temp1)/2.d0
         fdd1   = (fdd1   + temp2)/2.d0
         fddd1  = (fddd1  + temp3)/2.d0
         fdddd1 = (fdddd1 + temp4)/2.d0
         fd2 = fd1
     .        - float(iradelta)    * fdd1 
     .        + float(iradelta)**2 * fddd1
     .        - float(iradelta)**3 * fdddd1
         fdd2 = fdd1
     .         - 2.d0 * float(iradelta)    * fddd1
     .         + 3.d0 * float(iradelta)**2 * fddd1
         fddd2 = fddd1
     .          - 3.d0 * float(iradelta)    * fdddd1
         fdddd2 = fdddd1
      end if
      return
      end

      subroutine radopp(fd, fdd, fddd, fdddd, r, del)

      real*8 	fd, fdd, fddd, fdddd, r, del, temp1, temp2, temp3, temp4

      temp1 = fd
     .       - fdd   * (r/del)
     .       + fddd  * (r/del)**2
     .       - fdddd * (r/del)**3
      temp2 = fdd/del
     .       - 2.d0 * fddd  * (r/del)    /del
     .       + 3.d0 * fdddd * (r/del)**2 / del
      temp3 = fddd / del**2
     .       - 3.d0 * fdddd * (r/del) / del**2
      temp4 = fdddd / del**3

      fd    = temp1
      fdd   = temp2
      fddd  = temp3
      fdddd = temp4

      return
      end

      subroutine outdopp(fd, fdd, fddd, fdddd, r_old, r_new, del)

      implicit none
      real*8 	fd, fdd, fddd, fdddd, r_old, r_new, del, temp1, temp2, temp3, temp4

      temp1 = fd
     .       + fdd   * ((r_new-r_old)/del)
     .       + fddd  * ((r_new-r_old)/del)**2
     .       + fdddd * ((r_new-r_old)/del)**3
      temp2 = fdd
     .       + 2.d0 * fddd  * ((r_new-r_old)/del)
     .       + 3.d0 * fdddd * ((r_new-r_old)/del)**2
      temp3 = fddd
     .       + 3.d0 * fdddd  * ((r_new-r_old)/del)
      temp4 = fdddd

      fd    = temp1
      fdd   = temp2
      fddd  = temp3
      fdddd = temp4

      return
      end

      real*4 function seconds(t0)
      real*4 t0
      real*8 secondo

      seconds = secondo(-1) - t0

      return
      end

      integer function lastnb(string)
      
      character*(*) string
      integer ln
      
      ln = len(string) 
      
      lastnb = 0
      do i = ln , 1 , -1
         if(string(i:i) .ne. ' ') then
            lastnb = i 
            return
         end if
      end do
      return 
      end
      Integer*4 Function NextPower(number)
      
c Function to get the next highest power of two abov
c argument.
      
      implicit none

      integer power, k, number
      
      k = 0
      power = 0
 1    if(power .lt. number) then
         k = k + 1
         power = 2 ** k
         goto 1
      end if
      NextPower = k
      return
      end
